-- Love3D : Perspective Demo 4 --


function love.load()
	monitorWidth = 1024
	monitorHeight = 768
	love.graphics.setMode(monitorWidth, monitorHeight, false, true, 3)
	
	numBalls = 20
	balls = {}
	
	for i=1, numBalls do
		ball = {}
		ball.x = 0 -- Please note, ball.x/ball.y is the position in what WE see
		ball.y = 0 -- ball.xPos/ball.yPos/ball.zPos is the position in 3D space
		ball.xPos = math.random() * monitorWidth/2 - monitorWidth/4
		ball.yPos = math.random() * monitorHeight/2 - monitorHeight/4
		ball.zPos = math.random() * 200 - 100
		ball.scale = 1 -- This is an important value for perspective (obviously)
		table.insert(balls, ball)
	end
	
	friction = 0.9995
	vpX = monitorWidth/2 --  These are the vanishing points, 
	vpY = monitorHeight/2 -- in this case, it is the center of the screen.
	fl = 250 -- This number is usually 200 - 300, to simulate perspective.
	
	mouse = {}
	mouse.x = 0
	mouse.y = 0
end

function love.update(dt)
	local angleX = (love.mouse.getX() - vpX) * (0.05 * dt)
	local angleY = (love.mouse.getY() - vpY) * (0.05 * dt)
	for i, v in ipairs(balls) do
		if v.zPos > fl * -1 then
			perspective2(v, true, v.xPos, v.yPos, v.zPos)
			rotateX(v, angleX)
			rotateY(v, angleY)
		end
	end
end

function love.draw()
	
	for i, v in ipairs(balls) do
		love.graphics.circle("fill", v.x, v.y, 20 * v.scale, 25)
	end
	if showGridLines then
		for i=0, 15 do
			love.graphics.line(line.x, line.y, vpX, vpY)
			line.xPos = (love.mouse.getX() - vpX) + 64 * i
		end
	end
end

function perspective(object, isScale)
	local scale = fl / (fl + object.zPos)
	if isScale then
		object.scale = scale
	end
	object.x = vpX + object.xPos * scale
	object.y = vpY + object.yPos * scale
end

function perspective2(object, isScale, xPos, yPos, zPos)
	local scale = fl / (fl + zPos)
	if isScale then
		object.scale = scale
	end
	object.x = vpX + xPos * scale
	object.y = vpY + yPos * scale
end

function rotateX(v, angleX)
	local cosX = math.cos(angleX)
	local sinX = math.sin(angleX)
	local y1 = v.yPos * cosX - v.zPos * sinX
	local z1 = v.zPos * cosX + v.yPos * sinX
	v.yPos = y1
	v.zPos = z1
end

function rotateY(v, angleY)
	local cosY = math.cos(angleY)
	local sinY = math.sin(angleY)
	local x1 = v.xPos * cosY - v.zPos * sinY
	local z1 = v.zPos * cosY + v.xPos * sinY
	v.xPos = x1
	v.zPos = z1
end